<?php  if (!defined('BASEPATH')) exit('No direct script access allowed');
/* Copyright (c) 2011, Sam Clarke 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 * 
 * Redistributions of source code must retain the above copyright notice, this list
 *	of conditions and the following disclaimer.
 * 
 * Redistributions in binary form must reproduce the above copyright notice, this
 *	list of conditions and the following disclaimer in the documentation and/or
 *	other materials provided with the distribution.
 * 
 * Neither the name of Sam Clarke nor the names of its contributors may be used to
 *	endorse or promote products derived from this software without specific
 *	prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

/**
 * Codeigniter Askimet
 * 
 * You will need an API key to be able to use this
 * library. You can get an API from {@link https://akismet.com/signup/ The Akismet website}
 * 
 * To install, place this file in the application/libraries directory.
 * 
 * To set up a default API key and blog URL to use put the following in the
 * application config.php file:
 * <code>
 * $config['akismet_api_key']  = '<your key here>';
 * $config['akismet_blog_url'] = 'http://www.example.com/';
 * </code>
 * 
 * Then just change the values to the API key and website URL.
 * 
 * Example usage:
 * <code>
 * <?php
 * $this->load->library('Akismet');
 * 
 * if($this->akismet->is_key_valid(array('key' => 'API-KEY-HERE')))
 * 	$this->akismet->set_key('API-KEY-HERE');
 * 
 * // the comment to check, see is_spam for full list
 * $comment = array(
 * 		'comment_author'       => 'Auther',
 * 		'comment_author_email' => 'nobody@example.com',
 * 		'comment_author_url'   => 'http://www.example.com/',
 * 		'comment_content'      => 'This is an example comment.'
 * 	);
 * 
 * // check the comment
 * if($this->akismet->is_spam($comment))
 * 	echo 'The comment is spam.';
 * 
 * // mark it as spam
 * $this->akismet->mark_spam($comment);
 * 
 * // mark it as ham
 * $this->akismet->mark_ham($comment);
 * ?>
 * </code>
 * 
 * @version 1.0
 * @author Sam <samclarke.com>
 * @license http://opensource.org/licenses/BSD-3-Clause BSD License
 */
class Akismet
{
	/**
	 * Askimet Codeigniter library version
	 */
	const VERSION = '1.0';
	
	/**
	 * Askimet API url
	 */
	const API_URL = 'rest.akismet.com';
	
	/**
	 * Akimet API key
	 * @var string
	 */
	private $api_key;
	
	/**
	 * Default blog_url to use. Set in the config.
	 * @var string
	 */
	private $blog_url;
	
	
	public function __construct($api_key=null)
	{
		// load default api key and blog url from config
		$ci =& get_instance();
		$this->api_key  = $ci->config->item('akismet_api_key');
		$this->blog_url = $ci->config->item('akismet_blog_url');

		if($api_key !== null)
			$this->api_key = $api_key;
	}
	
	/**
	 * Checks if a comment is spam.
	 * 
	 * $comment_data should be an assoc array containing the following:
	 *	- blog (defaults to site_url('/'))
	 *		The front page or home URL of the instance making the request. For a blog or wiki this would be the front page. Note: Must be a full URI, including http://.
	 *	- user_ip (will try to detect the correct IP if not specified)
	 *		IP address of the comment submitter.
	 *	- user_agent (defaults to request user agaent)
	 *		User agent string of the web browser submitting the comment - typically the HTTP_USER_AGENT cgi variable. Not to be confused with the user agent of your Akismet library.
	 *	- referrer (defaults to request referer)
	 *		The content of the HTTP_REFERER header should be sent here.
	 *	- permalink
	 *		The permanent location of the entry the comment was submitted to.
	 *	- comment_type
	 *		May be blank, comment, trackback, pingback, or a made up value like "registration".
	 *	- comment_author
	 *		Name submitted with the comment
	 *	- comment_author_email
	 *		Email address submitted with the comment
	 *	- comment_author_url
	 *		URL submitted with comment
	 *	- comment_content
	 *		The content that was submitted.
	 * 
	 * @param array $comment_data
	 * @return bool True if is spam and false if not
	 */
	public function is_spam(array $comment_data=array())
	{
		$this->fix_comment_data($comment_data);

		return !($this->request("http://{$this->api_key}." . Akismet::API_URL . '/1.1/comment-check', $comment_data) == 'false');
	}
	
	/**
	 * Alias ofis_spam
	 * 
	 * @see is_spam
	 * @param array $comment_data
	 * @return bool
	 */
	public function check($comment_data)
	{
		return $this->is_spam($comment_data);
	}
	
	/**
	 * Checks if a key is valid.
	 * 
	 * $key_data should be in the following format:
	 *	- blog (defaults to site_url('/'))
	 *		The front page or home URL of the instance making the request. For a blog or wiki this would be the front page. Note: Must be a full URI, including http://.
	 *
	 * @param array $key_data
	 * @return bool
	 */
	public function is_key_valid(array $key_data=array())
	{
		$key_data['blog'] = isset($key_data['blog'])?$key_data['blog']:site_url('/');
		$key_data['key']  = isset($key_data['key'])?$key_data['key']:$this->api_key;
		
		return ($this->request('http://' . Akismet::API_URL . '/1.1/verify-key', $key_data) == 'valid');
	}
	
	/**
	 * Marks a comment as ham.
	 * 
	 * Ham is a comment that was marked as spam incorrectly.
	 * 
	 * $comment_data shoudd be an assoc array containing the
	 * comments data. For more information see {@see is_spam}
	 * 
	 * @param array $comment_data 
	 */
	public function mark_ham($comment_data)
	{
		$this->fix_comment_data($comment_data);
		$this->request("http://{$this->api_key}." . Akismet::API_URL . '/1.1/submit-ham', $comment_data);
	}
	
	/**
	 * Marks a comment as spam.
	 * 
	 * $comment_data shoudd be an assoc array containing the
	 * comments data. For more information see {@see is_spam}
	 * 
	 * @param array $comment_data 
	 */
	public function mark_spam($comment_data)
	{
		$this->fix_comment_data($comment_data);
		$this->request("http://{$this->api_key}." . Akismet::API_URL . '/1.1/submit-spam', $comment_data);
	}
	
	/**
	 * Gets the API key currently being used
	 * @return string
	 */
	public function get_key()
	{
		return $this->api_key;
	}
	
	/**
	 * Changes the API key used
	 * @param string $key The new API key
	 */
	public function set_key($key, $only_set_if_valid=false, $blog_url=null)
	{
		$this->api_key = $key;
	}
	
	private function fix_comment_data(&$comment_data)
	{
		// set default blog url if not set from config
		if(empty($this->blog_url))
			$this->blog_url = site_url('/');
		
		// these are all required. Try do good defaults.
		$comment_data['blog'] = isset($comment_data['blog'])?$comment_data['blog']:$this->blog_url;
		$comment_data['user_ip'] = isset($comment_data['user_ip'])?$comment_data['user_ip']:$this->get_remote_ip();
		$comment_data['user_agent'] = isset($comment_data['user_agent'])?$comment_data['user_agent']:$_SERVER['HTTP_USER_AGENT'];
		
		// referrer is required so if not set, set it to empty
		if(!isset($comment_data['referrer']))
		{
			if(isset($_SERVER['HTTP_REFERER']))
				$comment_data['referrer'] = $_SERVER['HTTP_REFERER'];
			else
				$comment_data['referrer'] = '';
		}
	}
	
	/**
	 * Trys to get the correct remove IP.
	 * 
	 * @return string The remove IP address
	 */
	private function get_remote_ip()
	{
		// cloudflare
		if(isset($_SERVER['HTTP_CF_CONNECTING_IP']))
			return $_SERVER['HTTP_CF_CONNECTING_IP'];
		
		// for PHP behind a proxy
		if($_SERVER['REMOTE_ADDR'] == $_SERVER['SERVER_ADDR']
			&& isset($_SERVER['HTTP_X_FORWARDED_FOR']) && !empty($_SERVER['HTTP_X_FORWARDED_FOR']))
			return $_SERVER['HTTP_X_FORWARDED_FOR'];
		
		// HTTP_X_FORWARDED_FOR could be spoofed by setting the header.
		// so is not included.
		// 
		// If you trust HTTP_X_FORWARDED_FOR i.e.
		// because you have set up a proxy that users will go though and
		// which will populate it stopping spoofing then uncomment the
		// two lines below:
		//if(isset($_SERVER['HTTP_X_FORWARDED_FOR']) && !empty($_SERVER['HTTP_X_FORWARDED_FOR']))
		//	return $_SERVER['HTTP_X_FORWARDED_FOR'];
		
		return $_SERVER['REMOTE_ADDR'];
	}
	
	/**
	 * Sends a HTTP POST request to the specified path
	 * and returns the response.
	 * 
	 * @param string $url      API URL to send the request to
	 * @param array  $post_data Assoc array of post data
	 * @return string Returns string result of boolean false
	 */
	private function request($url, $post_data)
	{
		if(empty($this->api_key))
		{
			log_message('error', 'No Akismet API set. Lookup failed.');
			return false;
		}
			
		$params = array(
			'http' => array(
				'method'  => 'POST',
				'header'  => "Content-type: application/x-www-form-urlencoded\r\n".
						'User-Agent: CodeIgniter/' . CI_VERSION . ' | SamsCIAskimetLib/' . Akismet::VERSION . "\r\n",
				'content' => http_build_query($post_data),
				'timeout' => 5
			));
		
		$context = stream_context_create($params);
		return file_get_contents($url, false, $context);
	}
}